package com.xjrsoft.module.textbook.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.xjrsoft.common.constant.GlobalConstant;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.xjrsoft.common.page.ConventPage;
import com.xjrsoft.common.page.PageOutput;
import com.xjrsoft.common.model.result.RT;
import com.xjrsoft.common.utils.VoToColumnUtil;
import com.xjrsoft.module.textbook.dto.AddTextbookIssueRecordDto;
import com.xjrsoft.module.textbook.dto.UpdateTextbookIssueRecordDto;
import cn.dev33.satoken.annotation.SaCheckPermission;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.support.ExcelTypeEnum;
import org.springframework.http.ResponseEntity;
import java.io.ByteArrayOutputStream;
import java.util.ArrayList;

import com.xjrsoft.module.textbook.dto.TextbookIssueRecordPageDto;
import com.xjrsoft.module.textbook.entity.TextbookIssueRecord;
import com.xjrsoft.module.textbook.service.ITextbookIssueRecordService;
import com.xjrsoft.module.textbook.vo.TextbookIssueRecordPageVo;

import com.xjrsoft.module.textbook.vo.TextbookIssueRecordVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
* @title: 教材出库记录
* @Author szs
* @Date: 2023-12-27
* @Version 1.0
*/
@RestController
@RequestMapping("/textbook" + "/textbookIssueRecord")
@Api(value = "/textbook"  + "/textbookIssueRecord",tags = "教材出库记录代码")
@AllArgsConstructor
public class TextbookIssueRecordController {


    private final ITextbookIssueRecordService textbookIssueRecordService;

    @GetMapping(value = "/page")
    @ApiOperation(value="教材出库记录列表(分页)")
    @SaCheckPermission("textbookissuerecord:detail")
    public RT<PageOutput<TextbookIssueRecordPageVo>> page(@Valid TextbookIssueRecordPageDto dto){

        LambdaQueryWrapper<TextbookIssueRecord> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper
                    .orderByDesc(TextbookIssueRecord::getId)
                .select(TextbookIssueRecord.class,x -> VoToColumnUtil.fieldsToColumns(TextbookIssueRecordPageVo.class).contains(x.getProperty()));
        IPage<TextbookIssueRecord> page = textbookIssueRecordService.page(ConventPage.getPage(dto), queryWrapper);
        PageOutput<TextbookIssueRecordPageVo> pageOutput = ConventPage.getPageOutput(page, TextbookIssueRecordPageVo.class);
        return RT.ok(pageOutput);
    }

    @GetMapping(value = "/info")
    @ApiOperation(value="根据id查询教材出库记录信息")
    @SaCheckPermission("textbookissuerecord:detail")
    public RT<TextbookIssueRecordVo> info(@RequestParam Long id){
        TextbookIssueRecord textbookIssueRecord = textbookIssueRecordService.getById(id);
        if (textbookIssueRecord == null) {
           return RT.error("找不到此数据！");
        }
        return RT.ok(BeanUtil.toBean(textbookIssueRecord, TextbookIssueRecordVo.class));
    }


    @PostMapping
    @ApiOperation(value = "新增教材出库记录")
    @SaCheckPermission("textbookissuerecord:add")
    public RT<Boolean> add(@Valid @RequestBody AddTextbookIssueRecordDto dto){
        TextbookIssueRecord textbookIssueRecord = BeanUtil.toBean(dto, TextbookIssueRecord.class);
        boolean isSuccess = textbookIssueRecordService.save(textbookIssueRecord);
    return RT.ok(isSuccess);
    }

    @PutMapping
    @ApiOperation(value = "修改教材出库记录")
    @SaCheckPermission("textbookissuerecord:edit")
    public RT<Boolean> update(@Valid @RequestBody UpdateTextbookIssueRecordDto dto){

        TextbookIssueRecord textbookIssueRecord = BeanUtil.toBean(dto, TextbookIssueRecord.class);
        return RT.ok(textbookIssueRecordService.updateById(textbookIssueRecord));

    }

    @DeleteMapping
    @ApiOperation(value = "删除教材出库记录")
    @SaCheckPermission("textbookissuerecord:delete")
    public RT<Boolean> delete(@Valid @RequestBody List<Long> ids){
        return RT.ok(textbookIssueRecordService.removeBatchByIds(ids));

    }

    @GetMapping("/export")
    @ApiOperation(value = "导出")
    public ResponseEntity<byte[]> exportData(@Valid TextbookIssueRecordPageDto dto, @RequestParam(defaultValue = "false") Boolean isTemplate) {
        List<TextbookIssueRecordPageVo> customerList = isTemplate != null && isTemplate ? new ArrayList<>() : ((PageOutput<TextbookIssueRecordPageVo>) page(dto).getData()).getList();
        ByteArrayOutputStream bot = new ByteArrayOutputStream();
        EasyExcel.write(bot, TextbookIssueRecordPageVo.class).automaticMergeHead(false).excelType(ExcelTypeEnum.XLSX).sheet().doWrite(customerList);

        return RT.fileStream(bot.toByteArray(), "TextbookIssueRecord" + ExcelTypeEnum.XLSX.getValue());
    }
}