package com.xjrsoft.module.textbook.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.xjrsoft.common.constant.GlobalConstant;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.xjrsoft.common.page.ConventPage;
import com.xjrsoft.common.page.PageOutput;
import com.xjrsoft.common.model.result.RT;
import com.xjrsoft.common.utils.VoToColumnUtil;
import com.xjrsoft.module.textbook.dto.AddTextbookStudentClaimDto;
import com.xjrsoft.module.textbook.dto.UpdateTextbookStudentClaimDto;
import cn.dev33.satoken.annotation.SaCheckPermission;

import com.xjrsoft.module.textbook.dto.TextbookStudentClaimPageDto;
import com.xjrsoft.module.textbook.entity.TextbookStudentClaim;
import com.xjrsoft.module.textbook.service.ITextbookStudentClaimService;
import com.xjrsoft.module.textbook.vo.TextbookStudentClaimPageVo;

import com.xjrsoft.module.textbook.vo.TextbookStudentClaimVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
* @title: 学生教材认领记录
* @Author szs
* @Date: 2023-12-26
* @Version 1.0
*/
@RestController
@RequestMapping("/textbook" + "/textbookStudentClaim")
@Api(value = "/textbook"  + "/textbookStudentClaim",tags = "学生教材认领记录代码")
@AllArgsConstructor
public class TextbookStudentClaimController {


    private final ITextbookStudentClaimService textbookStudentClaimService;

    @GetMapping(value = "/page")
    @ApiOperation(value="学生教材认领记录列表(分页)")
    @SaCheckPermission("textbookstudentclaim:detail")
    public RT<PageOutput<TextbookStudentClaimPageVo>> page(@Valid TextbookStudentClaimPageDto dto){

        LambdaQueryWrapper<TextbookStudentClaim> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper
                    .orderByDesc(TextbookStudentClaim::getId)
                .select(TextbookStudentClaim.class,x -> VoToColumnUtil.fieldsToColumns(TextbookStudentClaimPageVo.class).contains(x.getProperty()));
        IPage<TextbookStudentClaim> page = textbookStudentClaimService.page(ConventPage.getPage(dto), queryWrapper);
        PageOutput<TextbookStudentClaimPageVo> pageOutput = ConventPage.getPageOutput(page, TextbookStudentClaimPageVo.class);
        return RT.ok(pageOutput);
    }

    @GetMapping(value = "/info")
    @ApiOperation(value="根据id查询学生教材认领记录信息")
    @SaCheckPermission("textbookstudentclaim:detail")
    public RT<TextbookStudentClaimVo> info(@RequestParam Long id){
        TextbookStudentClaim textbookStudentClaim = textbookStudentClaimService.getById(id);
        if (textbookStudentClaim == null) {
           return RT.error("找不到此数据！");
        }
        return RT.ok(BeanUtil.toBean(textbookStudentClaim, TextbookStudentClaimVo.class));
    }


    @PostMapping
    @ApiOperation(value = "新增学生教材认领记录")
    @SaCheckPermission("textbookstudentclaim:add")
    public RT<Boolean> add(@Valid @RequestBody AddTextbookStudentClaimDto dto){
        TextbookStudentClaim textbookStudentClaim = BeanUtil.toBean(dto, TextbookStudentClaim.class);
        boolean isSuccess = textbookStudentClaimService.save(textbookStudentClaim);
    return RT.ok(isSuccess);
    }

    @PutMapping
    @ApiOperation(value = "修改学生教材认领记录")
    @SaCheckPermission("textbookstudentclaim:edit")
    public RT<Boolean> update(@Valid @RequestBody UpdateTextbookStudentClaimDto dto){

        TextbookStudentClaim textbookStudentClaim = BeanUtil.toBean(dto, TextbookStudentClaim.class);
        return RT.ok(textbookStudentClaimService.updateById(textbookStudentClaim));

    }

    @DeleteMapping
    @ApiOperation(value = "删除学生教材认领记录")
    @SaCheckPermission("textbookstudentclaim:delete")
    public RT<Boolean> delete(@Valid @RequestBody List<Long> ids){
        return RT.ok(textbookStudentClaimService.removeBatchByIds(ids));

    }

}