package com.xjrsoft.module.textbook.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.xjrsoft.common.constant.GlobalConstant;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.xjrsoft.common.page.ConventPage;
import com.xjrsoft.common.page.PageOutput;
import com.xjrsoft.common.model.result.RT;
import com.xjrsoft.common.utils.VoToColumnUtil;
import com.xjrsoft.module.textbook.dto.AddWfExerciseBookDto;
import com.xjrsoft.module.textbook.dto.UpdateWfExerciseBookDto;
import cn.dev33.satoken.annotation.SaCheckPermission;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.support.ExcelTypeEnum;
import org.springframework.http.ResponseEntity;
import java.io.ByteArrayOutputStream;
import java.util.ArrayList;

import com.xjrsoft.module.textbook.dto.WfExerciseBookPageDto;
import com.xjrsoft.module.textbook.entity.WfExerciseBook;
import com.xjrsoft.module.textbook.service.IWfExerciseBookService;
import com.xjrsoft.module.textbook.vo.WfExerciseBookPageVo;

import com.xjrsoft.module.textbook.vo.WfExerciseBookVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
* @title: 作业本征订
* @Author szs
* @Date: 2023-12-27
* @Version 1.0
*/
@RestController
@RequestMapping("/textbook" + "/wfExerciseBook")
@Api(value = "/textbook"  + "/wfExerciseBook",tags = "作业本征订代码")
@AllArgsConstructor
public class WfExerciseBookController {


    private final IWfExerciseBookService wfExerciseBookService;

    @GetMapping(value = "/page")
    @ApiOperation(value="作业本征订列表(分页)")
    @SaCheckPermission("wfexercisebook:detail")
    public RT<PageOutput<WfExerciseBookPageVo>> page(@Valid WfExerciseBookPageDto dto){

        LambdaQueryWrapper<WfExerciseBook> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper
                    .orderByDesc(WfExerciseBook::getId)
                .select(WfExerciseBook.class,x -> VoToColumnUtil.fieldsToColumns(WfExerciseBookPageVo.class).contains(x.getProperty()));
        IPage<WfExerciseBook> page = wfExerciseBookService.page(ConventPage.getPage(dto), queryWrapper);
        PageOutput<WfExerciseBookPageVo> pageOutput = ConventPage.getPageOutput(page, WfExerciseBookPageVo.class);
        return RT.ok(pageOutput);
    }

    @GetMapping(value = "/info")
    @ApiOperation(value="根据id查询作业本征订信息")
    @SaCheckPermission("wfexercisebook:detail")
    public RT<WfExerciseBookVo> info(@RequestParam Long id){
        WfExerciseBook wfExerciseBook = wfExerciseBookService.getByIdDeep(id);
        if (wfExerciseBook == null) {
           return RT.error("找不到此数据！");
        }
        return RT.ok(BeanUtil.toBean(wfExerciseBook, WfExerciseBookVo.class));
    }


    @PostMapping
    @ApiOperation(value = "新增作业本征订")
    @SaCheckPermission("wfexercisebook:add")
    public RT<Boolean> add(@Valid @RequestBody AddWfExerciseBookDto dto){
        WfExerciseBook wfExerciseBook = BeanUtil.toBean(dto, WfExerciseBook.class);
        boolean isSuccess = wfExerciseBookService.add(wfExerciseBook);
    return RT.ok(isSuccess);
    }

    @PutMapping
    @ApiOperation(value = "修改作业本征订")
    @SaCheckPermission("wfexercisebook:edit")
    public RT<Boolean> update(@Valid @RequestBody UpdateWfExerciseBookDto dto){

        WfExerciseBook wfExerciseBook = BeanUtil.toBean(dto, WfExerciseBook.class);
        return RT.ok(wfExerciseBookService.update(wfExerciseBook));

    }

    @DeleteMapping
    @ApiOperation(value = "删除作业本征订")
    @SaCheckPermission("wfexercisebook:delete")
    public RT<Boolean> delete(@Valid @RequestBody List<Long> ids){
        return RT.ok(wfExerciseBookService.delete(ids));

    }

    @GetMapping("/export")
    @ApiOperation(value = "导出")
    public ResponseEntity<byte[]> exportData(@Valid WfExerciseBookPageDto dto, @RequestParam(defaultValue = "false") Boolean isTemplate) {
        List<WfExerciseBookPageVo> customerList = isTemplate != null && isTemplate ? new ArrayList<>() : ((PageOutput<WfExerciseBookPageVo>) page(dto).getData()).getList();
        ByteArrayOutputStream bot = new ByteArrayOutputStream();
        EasyExcel.write(bot, WfExerciseBookPageVo.class).automaticMergeHead(false).excelType(ExcelTypeEnum.XLSX).sheet().doWrite(customerList);

        return RT.fileStream(bot.toByteArray(), "WfExerciseBook" + ExcelTypeEnum.XLSX.getValue());
    }
}