package com.xjrsoft.module.textbook.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.xjrsoft.common.constant.GlobalConstant;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.xjrsoft.common.page.ConventPage;
import com.xjrsoft.common.page.PageOutput;
import com.xjrsoft.common.model.result.RT;
import com.xjrsoft.common.utils.VoToColumnUtil;
import com.xjrsoft.module.textbook.dto.AddWfTextbookSubscriptionDto;
import com.xjrsoft.module.textbook.dto.UpdateWfTextbookSubscriptionDto;
import cn.dev33.satoken.annotation.SaCheckPermission;

import com.xjrsoft.module.textbook.dto.WfTextbookSubscriptionPageDto;
import com.xjrsoft.module.textbook.entity.WfTextbookSubscription;
import com.xjrsoft.module.textbook.service.IWfTextbookSubscriptionService;
import com.xjrsoft.module.textbook.vo.WfTextbookSubscriptionPageVo;

import com.xjrsoft.module.textbook.vo.WfTextbookSubscriptionVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
* @title: 教材教辅征订
* @Author szs
* @Date: 2023-12-25
* @Version 1.0
*/
@RestController
@RequestMapping("/textbook" + "/wfTextbookSubscription")
@Api(value = "/textbook"  + "/wfTextbookSubscription",tags = "教材教辅征订代码")
@AllArgsConstructor
public class WfTextbookSubscriptionController {


    private final IWfTextbookSubscriptionService wfTextbookSubscriptionService;

    @GetMapping(value = "/page")
    @ApiOperation(value="教材教辅征订列表(分页)")
    @SaCheckPermission("wftextbooksubscription:detail")
    public RT<PageOutput<WfTextbookSubscriptionPageVo>> page(@Valid WfTextbookSubscriptionPageDto dto){

        LambdaQueryWrapper<WfTextbookSubscription> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper
                    .orderByDesc(WfTextbookSubscription::getId)
                .select(WfTextbookSubscription.class,x -> VoToColumnUtil.fieldsToColumns(WfTextbookSubscriptionPageVo.class).contains(x.getProperty()));
        IPage<WfTextbookSubscription> page = wfTextbookSubscriptionService.page(ConventPage.getPage(dto), queryWrapper);
        PageOutput<WfTextbookSubscriptionPageVo> pageOutput = ConventPage.getPageOutput(page, WfTextbookSubscriptionPageVo.class);
        return RT.ok(pageOutput);
    }

    @GetMapping(value = "/info")
    @ApiOperation(value="根据id查询教材教辅征订信息")
    @SaCheckPermission("wftextbooksubscription:detail")
    public RT<WfTextbookSubscriptionVo> info(@RequestParam Long id){
        WfTextbookSubscription wfTextbookSubscription = wfTextbookSubscriptionService.getByIdDeep(id);
        if (wfTextbookSubscription == null) {
           return RT.error("找不到此数据！");
        }
        return RT.ok(BeanUtil.toBean(wfTextbookSubscription, WfTextbookSubscriptionVo.class));
    }


    @PostMapping
    @ApiOperation(value = "新增教材教辅征订")
    @SaCheckPermission("wftextbooksubscription:add")
    public RT<Boolean> add(@Valid @RequestBody AddWfTextbookSubscriptionDto dto){
        WfTextbookSubscription wfTextbookSubscription = BeanUtil.toBean(dto, WfTextbookSubscription.class);
        boolean isSuccess = wfTextbookSubscriptionService.add(wfTextbookSubscription);
    return RT.ok(isSuccess);
    }

    @PutMapping
    @ApiOperation(value = "修改教材教辅征订")
    @SaCheckPermission("wftextbooksubscription:edit")
    public RT<Boolean> update(@Valid @RequestBody UpdateWfTextbookSubscriptionDto dto){

        WfTextbookSubscription wfTextbookSubscription = BeanUtil.toBean(dto, WfTextbookSubscription.class);
        return RT.ok(wfTextbookSubscriptionService.update(wfTextbookSubscription));

    }

    @DeleteMapping
    @ApiOperation(value = "删除教材教辅征订")
    @SaCheckPermission("wftextbooksubscription:delete")
    public RT<Boolean> delete(@Valid @RequestBody List<Long> ids){
        return RT.ok(wfTextbookSubscriptionService.delete(ids));

    }

}